<?php

/********************************************************************
 *   Autori:             Marco Colombo (marco@openit.it)
 *                       Valentina Portolan (valentina@openit.it)
 *                       Giuliano Natali Diaolin (diaolin@openit.it)
 *  Copyright:           Open It S.r.l.
 *                       Viale Dante, 78
 *                       38057 Pergine Valsugana (TN) ITALY
 *                       Tel: +39 0461 534800 Fax: +39 0461 538443
 *********************************************************************/
   
/**
 * abook_database.php
 *
 * Copyright (c) 1999-2003 The SquirrelMail Project Team
 * Licensed under the GNU GPL. For full terms see the file COPYING.
 *
 * Backend for personal addressbook stored in a database,
 * accessed using the DB-classes in PEAR.
 *
 * IMPORTANT:  The PEAR modules must be in the include path
 * for this class to work.
 *
 * An array with the following elements must be passed to
 * the class constructor (elements marked ? are optional):
 *
 *    dsn       => database DNS (see PEAR for syntax)
 *    table     => table to store addresses in (must exist)
 *    owner     => current user (owner of address data)
 *  ? writeable => set writeable flag (true/false)
 *
 * The table used should have the following columns:
 * owner, nickname, firstname, lastname, email, label
 * The pair (owner,nickname) should be unique (primary key).
 *
 *  NOTE. This class should not be used directly. Use the
 *        "AddressBook" class instead.
 *
 * $Id: abook_database_global.php,v 1.7 2004/06/09 09:00:36 marco Exp $
 */

if (!defined('SM_PATH')) { define('SM_PATH','../'); }

require_once( SM_PATH . 'database/db_connect.class.php' );
   
class abook_database_global extends addressbook_backend {
    var $btype = 'global';
    var $bname = 'database';
      
    var $dsn       = '';
    var $table     = '';
    var $owner     = '';
    var $dbh       = false;
      
    var $writeable = true;
      
    /* ========================== Private ======================= */
      
    /* Constructor */
    function abook_database_global($param) {
	global $data_dir;
	$this->sname = _("Global address book"); 
         
        if (is_array($param)) {
            if (empty($param['dsn']) || 
                empty($param['table'])) {
                return $this->set_error('Invalid parameters');
            }
            
            $this->dsn   = $param['dsn'];
            $this->table = $param['table'];
            
            if (!empty($param['name'])) {
               $this->sname = $param['name'];
            }

            if (isset($param['writeable'])) {
               $this->writeable = $param['writeable'];
            }

    	    $this->open(true);
        } else {
            return $this->set_error('Invalid argument to constructor');
        }
    }
      
      
    /* Open the database. New connection if $new is true */
    function open($new = false) {
        $this->error = '';

        /* Return true is file is open and $new is unset */
        if ($this->dbh && !$new) { return true; }

        /* Close old file, if any */
        if ($this->dbh) { $this->close(); }
        $dbh = & new db_connect();

        if ($dbh->error) {
            return $this->set_error(sprintf(_("Database error: %s"),$dbh->error));
        }

        $this->dbh = $dbh;
        return true;
    }

    /* Close the file and forget the filehandle */
    function close() { $this->dbh = false; }

    /* ========================== Public ======================== */
     
    /* Search the file */
    function &search($expr,$field) {
	global $fieldarrayabook;
        if(!$this->open()) { return false; }
         
        /* To be replaced by advanded search expression parsing */
        if (is_array($expr)) { return; }

        /* Make regexp from glob'ed expression  */
        $expr = str_replace('?', '_', $expr);
        $expr = str_replace('*', '%', $expr);
        $expr = "%$expr%";
	$table = $this->dbh->db_quotename($this->table);
	$where1 = $this->dbh->db_quotename("utente");
	$where2 = $this->dbh->db_quotename($field);
        $query = sprintf("SELECT * FROM %s WHERE %s='Public' AND %s LIKE '%s'",
                         $table, $where1, $where2, $expr);
        $res = $this->dbh->db_loadList( $query );
	$ret = array();
        foreach ($res as $row) {
	    $arow = array();
            if ($row['idrub']) {
		$arow = $row;
                $arow['backend'] = $this->bnum;
                $arow['source'] = &$this->sname;
		array_push($ret,$arow);
            }
        }

        return $ret;
    }

    /* Lookup alias */
    function test_id($userdata,$idrub=0) {
	global $fieldarrayabook;
        if (!empty($userdata['nickname'])) { $alias = $userdata['nickname']; }
        else { return false; }
        if (!$this->open()) { return true; }

	$table = $this->dbh->db_quotename($this->table);
	$where1 = $this->dbh->db_quotename("utente");
	$where2 = $this->dbh->db_quotename("nickname");
	$where3 = $this->dbh->db_quotename("idrub");

	$query = "SELECT * FROM $table WHERE $where1='Public' AND $where2='$alias'";
	if ($idrub) { $query .= " AND $where3 <> $idrub"; }

        $arow = $this->dbh->db_loadHash( $query );
        if (count($arow)) {
            $arow['backend'] = $this->bnum;
            $arow['source'] = &$this->sname;
        }
        return $arow;
    }
     
    /* Lookup alias */
    function &lookup($alias) {
	global $fieldarrayabook;
        if (empty($alias)) { return array(); }
         
        if (!$this->open()) { return false; }

	$table = $this->dbh->db_quotename($this->table);
	$where1 = $this->dbh->db_quotename("utente");
	$where2 = $this->dbh->db_quotename("nickname");

        $query = sprintf("SELECT * FROM %s WHERE %s='Public' AND %s='%s'",
                         $table, $where1, $where2, $alias);

        $arow = $this->dbh->db_loadHash( $query );
        if (count($arow)) {
            $arow['backend'] = $this->bnum;
            $arow['source'] = &$this->sname;
        }
        return $arow;

    }

    /* Lookup alias */
    function &lookup_id($alias) {
	global $fieldarrayabook;

        if (empty($alias)) { return array(); }
         
        if (!$this->open()) { return false; }

	$table = $this->dbh->db_quotename($this->table);
	$where1 = $this->dbh->db_quotename("idrub");

        $query = sprintf("SELECT * FROM %s WHERE %s=%s",
                         $table, $where1, $alias);

        $arow = $this->dbh->db_loadHash( $query );
        if (count($arow)) {
            $arow['backend'] = $this->bnum;
            $arow['source'] = &$this->sname;
        }
        return $arow;
    }

    /* Lookup contacts for group */
    function lookup_group_id($strings,$address = 0,$where="ALL") {
        global $fieldarrayabook;
	$where2 = $this->dbh->db_quotename("idrub");

        $ids = explode(",",$strings);
        $alias = implode("' OR $where2 = '",$ids);

        $this->open();

	$table = $this->dbh->db_quotename($this->table);
	$where1 = $this->dbh->db_quotename("utente");

        $query = sprintf("SELECT * FROM %s WHERE %s='Public' AND ( %s = '%s' )",
                         $table, $where1, $where2, $alias);

        $res = $this->dbh->db_loadList( $query );

        $arow = array();
        foreach ($res as $row) {
            if ($address == 1) {
                array_push($arow,'"'.$row["Nominativo"].'" <'.$row["Email"].'>');
            } else if ($address == 2) {
                $a = array();
                foreach ($fieldarrayabook as $k => $id) {
                    $a[$k] = $row[$k];
                }
                $a['backend'] = $this->bnum;
                $a['source'] = &$this->sname;
		if ($a['idrub'] && (($where != "ALL" && preg_match('/^'.$where.'/i',$a['Nominativo'])) || $where == "ALL")) {
                    array_push($arow,$a);
                }
            } else {
                $arow["2;".$row['idrub']] = $row['Nominativo'];
            }
        }

        return $arow;
    }

    /* List all addresses */
    function &list_addr($Email=0,$initial="ALL",$nsearch="") {
	global $fieldarrayabook;

        $ret = array();
        if (!$this->open()) { return false; }

        $ret = array();

	$table = $this->dbh->db_quotename($this->table);
	$where1 = $this->dbh->db_quotename("utente");
	$where2 = $this->dbh->db_quotename("Email");
	$where3 = $this->dbh->db_quotename("Nominativo");

	$query = "SELECT * FROM $table WHERE $where1='Public'";
	if ($Email == 1) {
	    $query .= " AND $where2 is not null";
	}
        if ($initial != "ALL") {
            if ($initial == "number") {
                $query .= " AND $where3 RLIKE '^[0-9]'";
            } else {
                $query .= " AND $where3 LIKE '".$initial."%'";
            }
        }
        if ($nsearch) {
            $query .= " AND $where3 LIKE '%".$nsearch."%'";
        }

        $res = $this->dbh->db_loadList( $query );

        foreach ($res as $row) {
            $row['backend'] = $this->bnum;
            $row['source'] = &$this->sname;
            if ($row['idrub']) { array_push($ret,$row); }
        }

        return $ret;
    }

    /* Add address */
    function add($userdata) {
	global $fieldarrayabook;
        if (!$this->writeable) {
            return $this->set_error(_("Addressbook is read-only"));
        }

        if (!$this->open()) { return false; }

        /* See if user exists already */
        $ret = $this->test_id($userdata);
        if(!empty($ret["idrub"])) {
            return $this->set_error(sprintf(_("Nickname '%s' already exist"),
                   $ret['nickname']));
        }

        $plus = array("utente" => "Public");
        $r = $this->dbh->db_insert( $this->table, $userdata, "idrub", $fieldarrayabook, $plus);

        if ($r == true) { return true; }
        /* Fail */
         return $this->set_error(sprintf(_("Database error: %s"),$r));

    }

    /* Delete address */
    function remove($alias) {
        if (!$this->writeable) {
            return $this->set_error(_("Addressbook is read-only"));
        }

        if (!$this->open()) { return false; }

        $r = $this->dbh->db_delete( $this->table, "idrub", $alias );

        if ($r == true) { return true; }

        /* Fail */
        return $this->set_error(sprintf(_("Database error: %s"),$r));
    }

    /* Modify address */
    function modify($alias, $userdata) {
	global $fieldarrayabook;

        if (!$this->writeable) {
            return $this->set_error(_("Addressbook is read-only"));
        }

        if (!$this->open()) { return false; }
         
        /* See if user exists */
        $ret = $this->lookup_id($alias);
        if(empty($ret["idrub"])) {
            return $this->set_error(sprintf(_("Contact '%s' does not exist"),
                $userdata['Nominativo']));
        }

        /* See if user exists already */
        $ret = $this->test_id($userdata,$alias);
        if(!empty($ret["idrub"])) {
            return $this->set_error(sprintf(_("Nickname '%s' already exist"),
                   $ret['nickname']));
        }

	$r = $this->dbh->db_update( $this->table, $userdata, "idrub", $alias, $fieldarrayabook );

        if ($r == true) { return true; }

        /* Fail */
        return $this->set_error(sprintf(_("Database error: %s"),$r));

    }
} /* End of class abook_database */

?>
