<?php

/********************************************************************
 *   Autori:             Marco Colombo (marco@openit.it)
 *                       Valentina Portolan (valentina@openit.it)
 *                       Giuliano Natali Diaolin (diaolin@openit.it)
 *  Copyright:           Open It S.r.l.
 *                       Viale Dante, 78
 *                       38057 Pergine Valsugana (TN) ITALY
 *                       Tel: +39 0461 534800 Fax: +39 0461 538443
 *********************************************************************/
   
/**
 * abook_database.php
 *
 * Copyright (c) 1999-2003 The SquirrelMail Project Team
 * Licensed under the GNU GPL. For full terms see the file COPYING.
 *
 * Backend for personal addressbook stored in a database,
 * accessed using the DB-classes in PEAR.
 *
 * IMPORTANT:  The PEAR modules must be in the include path
 * for this class to work.
 *
 * An array with the following elements must be passed to
 * the class constructor (elements marked ? are optional):
 *
 *    dsn       => database DNS (see PEAR for syntax)
 *    table     => table to store addresses in (must exist)
 *    owner     => current user (owner of address data)
 *  ? writeable => set writeable flag (true/false)
 *
 * The table used should have the following columns:
 * owner, nickname, firstname, lastname, email, label
 * The pair (owner,nickname) should be unique (primary key).
 *
 *  NOTE. This class should not be used directly. Use the
 *        "AddressBook" class instead.
 *
 * $Id: group_abook_database_global.php,v 1.3 2004/06/09 09:00:36 marco Exp $
 */

if (!defined('SM_PATH')) { define('SM_PATH','../'); }
   
require_once( SM_PATH . 'database/db_connect.class.php' );
   
class group_abook_database_global extends addressbook_backend {
    var $btype = 'global';
    var $bname = 'database';
      
    var $dsn       = '';
    var $table     = '';
    var $owner     = '';
    var $dbh       = false;
      
    var $writeable = true;
      
    /* ========================== Private ======================= */
      
    /* Constructor */
    function group_abook_database_global($param) {
	global $data_dir;
	$this->sname = _("Global group address book"); 
         
        if (is_array($param)) {
            if (empty($param['dsn']) || 
                empty($param['table'])) {
                return $this->set_error('Invalid parameters');
            }
            
            $this->dsn   = $param['dsn'];
            $this->table = $param['table'];
            
            if (!empty($param['name'])) {
               $this->sname = $param['name'];
            }

            if (isset($param['writeable'])) {
               $this->writeable = $param['writeable'];
            }

    	    $this->open(true);
        } else {
            return $this->set_error('Invalid argument to constructor');
        }
    }
      
      
    /* Open the database. New connection if $new is true */
    function open($new = false) {
        $this->error = '';

       /* Return true is file is open and $new is unset */
        if ($this->dbh && !$new) { return true; }

        /* Close old file, if any */
        if ($this->dbh) { $this->close(); }
        $dbh = & new db_connect();

        if ($dbh->error) {
            return $this->set_error(sprintf(_("Database error: %s"),$dbh->error));
        }

        $this->dbh = $dbh;
        return true;
    }

    /* Close the file and forget the filehandle */
    function close() { $this->dbh = false; }

    /* ========================== Public ======================== */
     
    /* Lookup alias */
    function test_id($userdata,$idrub=0) {
	global $fieldarraygroupabook;

        if (!empty($userdata['group_name'])) { $alias = $userdata['group_name']; }
        else { return false; }

        $table = $this->dbh->db_quotename($this->table);
        $where1 = $this->dbh->db_quotename("utente");
        $where2 = $this->dbh->db_quotename("group_name");
        $where3 = $this->dbh->db_quotename("group_id");

        $query = "SELECT * FROM $table WHERE $where1='Public' AND $where2='$alias'";
        if ($idrub) { $query .= " AND $where3 <> $idrub"; }

        $arow = $this->dbh->db_loadHash( $query );
	if (count($arow)) {
	    $arow['backend'] = $this->bnum;
    	    $arow['source'] = &$this->sname;
	}
        return $arow;
    }
     
    /* Lookup alias */
    function &lookup($alias) {
        global $fieldarraygroupabook;
        if (!$alias) { return array(''); }

        if (!$this->open()) { return false; }

        $table = $this->dbh->db_quotename($this->table);
        $where1 = $this->dbh->db_quotename("utente");
        $where2 = $this->dbh->db_quotename("group_name");

        $query = sprintf("SELECT * FROM %s WHERE %s='Public' AND %s='%s'",
                         $table, $where1, $where2, $alias);

        $arow = $this->dbh->db_loadHash( $query );
	if (count($arow)) {
    	    $arow['backend'] = $this->bnum;
    	    $arow['source'] = &$this->sname;
	}
        return $arow;
    }

    /* Lookup alias */
    function &lookup_id($alias) {
        global $fieldarraygroupabook;

        if (!$this->open()) { return false; }

        $table = $this->dbh->db_quotename($this->table);
        $where1 = $this->dbh->db_quotename("utente");
        $where2 = $this->dbh->db_quotename("group_id");

        $query = sprintf("SELECT * FROM %s WHERE %s='Public' AND %s=%s",
                         $table, $where1, $where2, $alias);

        $arow = $this->dbh->db_loadHash( $query );
	if (count($arow)) {
    	    $arow['backend'] = $this->bnum;
    	    $arow['source'] = &$this->sname;
	}
        return $arow;
    }

    /* List all addresses */
    function &list_addr() {
        global $fieldarraygroupabook;
        $ret = array();
        $this->open();

        $table = $this->dbh->db_quotename($this->table);
        $where1 = $this->dbh->db_quotename("utente");

        $query = "SELECT * FROM $table WHERE $where1='Public'";

        $res = $this->dbh->db_loadList( $query );

        foreach ($res as $row) {
            $row['backend'] = $this->bnum;
            $row['source'] = &$this->sname;
            if ($row['group_id']) { array_push($ret,$row); }
        }

        return $ret;
    }

    /* Add address */
    function add($userdata) {
	global $fieldarraygroupabook;
        if (!$this->writeable) {
            return $this->set_error(_("Group Addressbook is read-only"));
        }

        if (!$this->open()) { return false; }

        /* See if user exists already */
        $ret = $this->test_id($userdata);
        if(!empty($ret["group_id"])) {
            return $this->set_error(sprintf(_("Group Name '%s' already exist"),
                   $ret['group_name']));
        }

        $plus = array("utente" => "Public");
        $r = $this->dbh->db_insert( $this->table, $userdata, "group_id", $fieldarraygroupabook, $plus);

        if ($r == true) { return true; }
        /* Fail */
        return $this->set_error(sprintf(_("Database error: %s"),$r));
    }

    /* Delete address */
    function remove($alias) {
        if (!$this->writeable) {
            return $this->set_error(_("Addressbook is read-only"));
        }

        if (!$this->open()) { return false; }

        $r = $this->dbh->db_delete( $this->table, "group_id", $alias );

        if ($r == true) { return true; }

        /* Fail */
        return $this->set_error(sprintf(_("Database error: %s"),$r));
    }

    /* Delete address */
    function remove_contacts($alias) {
        if (!$this->writeable) {
            return $this->set_error(_("Addressbook is read-only"));
        }

        if (!$this->open()) { return false; }

        $key = "group_list_public";

        $table = $this->dbh->db_quotename($this->table);
        $where1 = $this->dbh->db_quotename("utente");

        $query = "SELECT * FROM $table WHERE $where1='Public'";

        $res = $this->dbh->db_loadList( $query );

        foreach ($res as $row) {
            $ids = $row[$key];
            $ids = ",".$ids.",";
            if( preg_match("/,".$alias.",/",$ids)) {
                $ids = preg_replace('/'.$alias.',/','',$ids);
                $ids = preg_replace('/^,/','',$ids);
                $ids = preg_replace('/,$/','',$ids);
                $where = array(group_id => $row['group_id'], utente => "Public");
                $r = $this->dbh->db_update_field( $this->table, $key, $ids, $where );
                if ($r != true) { $this->error .= $r; }
            }
        }
    }

    /* Modify address */
    function modify($alias, $userdata) {
	global $fieldarraygroupabook;
        if (!$this->writeable) {
            return $this->set_error(_("Addressbook is read-only"));
        }

        if (!$this->open()) { return false; }
         
        /* See if user exists */
        $ret = $this->lookup_id($alias);
        if(empty($ret["group_id"])) {
            return $this->set_error(sprintf(_("Group '%s' does not exist"),
                $userdata['group_name']));
        }

        /* See if user exists already */
        $ret = $this->test_id($userdata,$alias);
        if(!empty($ret["group_id"])) {
            return $this->set_error(sprintf(_("Group Name '%s' already exist"),
                   $ret['group_name']));
        }

        $r = $this->dbh->db_update( $this->table, $userdata, "group_id", $alias, $fieldarraygroupabook );

        if ($r == true) { return true; }

        /* Fail */
        return $this->set_error(sprintf(_("Database error: %s"),$r));
    }
} /* End of class abook_database */

?>
