<?php

/********************************************************************
 *   Autori:             Marco Colombo (marco@openit.it)
 *                       Valentina Portolan (valentina@openit.it)
 *                       Giuliano Natali Diaolin (diaolin@openit.it)
 *  Copyright:           Open It S.r.l.
 *                       Viale Dante, 78
 *                       38057 Pergine Valsugana (TN) ITALY
 *                       Tel: +39 0461 534800 Fax: +39 0461 538443
 *********************************************************************/

/**
 * tree.php
 *
 * Copyright (c) 1999-2003 The SquirrelMail Project Team
 * Licensed under the GNU GPL. For full terms see the file COPYING.
 *
 * This code provides various string manipulation functions that are
 * used by the rest of the Squirrelmail code.
 *
 * $Id: tree.php,v 1.1 2004/02/02 14:00:13 marco Exp $
 */

if (!defined('SM_PATH')) { define('SM_PATH','../'); }

require_once(SM_PATH . 'functions/imap.php');
require_once(SM_PATH . 'functions/global.php');

/**
 * findParentForChild
 *
 * Recursive function to find the correct parent for a new node
 */

function findParentForChild($value, $treeIndexToStart, $tree) {
    // is $value in $tree[$treeIndexToStart]['value']
    if ((isset($tree[$treeIndexToStart])) && (strstr($value, $tree[$treeIndexToStart]['value']))) {
        // do I have children, if not then must be a childnode of the current node
        if ($tree[$treeIndexToStart]['doIHaveChildren']) {
            // loop through each subNode checking to see if we are a subNode of one of them
            for ($i=0;$i< count($tree[$treeIndexToStart]['subNodes']);$i++) {
            $result = findParentForChild($value, $tree[$treeIndexToStart]['subNodes'][$i], $tree);
            if ($result > -1)
                return $result;
            }
            // if we aren't a child of one of the subNodes, must be a child of current node
            return $treeIndexToStart;
        } else
            return $treeIndexToStart;
    } else {
        // we aren't a child of this node at all
        return -1;
    }
}

function addChildNodeToTree($comparisonValue, $value, &$tree) {
    $parentNode = findParentForChild($comparisonValue, 0, $tree);

    // create a new subNode
    $newNodeIndex = count($tree);
    $tree[$newNodeIndex]['value'] = $value;
    $tree[$newNodeIndex]['doIHaveChildren'] = false;

    if ($tree[$parentNode]['doIHaveChildren'] == false) {
        // make sure the parent knows it has children
        $tree[$parentNode]['subNodes'][0] = $newNodeIndex;
        $tree[$parentNode]['doIHaveChildren'] = true;
    } else {
        $nextSubNode = count($tree[$parentNode]['subNodes']);
        // make sure the parent knows it has children
        $tree[$parentNode]['subNodes'][$nextSubNode] = $newNodeIndex;
    }
}

function walkTreeInPreOrderEmptyTrash($index, $imap_stream, $tree) {
    global $trash_folder;
    if ($tree[$index]['doIHaveChildren']) {
        for ($j = 0; $j < count($tree[$index]['subNodes']); $j++) {
            walkTreeInPreOrderEmptyTrash($tree[$index]['subNodes'][$j], $imap_stream, $tree);
        }
    }
    if ($tree[$index]['value'] != $trash_folder) {
        sqimap_mailbox_delete($imap_stream, $tree[$index]['value']);
    } else {
        $mbx_response = sqimap_mailbox_select($imap_stream, $trash_folder);
        if ($mbx_response['EXISTS'] > 0) {
            sqimap_messages_flag ($imap_stream, 1, '*', 'Deleted', true);
            // CLOSE === EXPUNGE and UNSELECT
            sqimap_run_command($imap_stream,'CLOSE',false,$response,$message);
        }
    }
}

function walkTreeInPreOrderDeleteFolders($index, $imap_stream, $tree) {
    if ($tree[$index]['doIHaveChildren']) {
        for ($j = 0; $j < count($tree[$index]['subNodes']); $j++) {
            walkTreeInPreOrderDeleteFolders($tree[$index]['subNodes'][$j], $imap_stream, $tree);
        }
    }
    sqimap_mailbox_delete($imap_stream, $tree[$index]['value']);
}

function walkTreeInPostOrderCreatingFoldersUnderTrash($index, $imap_stream, $tree, $topFolderName, $norewind = 0) {
    global $trash_folder, $delimiter, $default_folder_prefix;
    $myname = ereg_replace( "^$default_folder_prefix", '', $tree[$index]['value']);
    $path_array = explode($delimiter,$myname);
    $mypath = $trash_folder;
    for ($i=0;$i<count($path_array);$i++) {
	if (!$norewind) {
	    if (!sqimap_mailbox_exists($imap_stream, $mypath . $delimiter . $path_array[$i])) {
		sqimap_mailbox_create($imap_stream, $mypath . $delimiter . $path_array[$i], '');
	    }
	}
	$mypath .= $delimiter . $path_array[$i];
    }
    if ($norewind == 1) {
	if (!sqimap_mailbox_exists($imap_stream, $mypath )) {
	    sqimap_mailbox_create($imap_stream, $mypath, '');
	}
    }
    $mbx_response = sqimap_mailbox_select($imap_stream, $tree[$index]['value']);
    $messageCount = $mbx_response['EXISTS'];
    if ($messageCount > 0) {
        sqimap_messages_copy($imap_stream, 1, '*', $mypath);
    }
    // after copy close the mailbox to get in unselected state
    sqimap_run_command($imap_stream,'CLOSE',false,$response,$message);
    // se ho figli rilancio me stesso sui figli
    if ($tree[$index]['doIHaveChildren']) {
        for ($j = 0;$j < count($tree[$index]['subNodes']); $j++) {
	    // con 1 finale non esegue la creazione delle direactory padri
            walkTreeInPostOrderCreatingFoldersUnderTrash($tree[$index]['subNodes'][$j], $imap_stream, $tree, $topFolderName, 1);
	}
    }
}

function walkTreeInPostOrderMoveMessagesUnderTrash($index, $imap_stream, $tree, $topFolderName) {
    global $trash_folder, $delimiter, $default_folder_prefix;
    $mbx_response = sqimap_mailbox_select($imap_stream, $tree[$index]['value']);
    $messageCount = $mbx_response['EXISTS'];
    if ($messageCount > 0) {
        sqimap_messages_copy($imap_stream, 1, '*', $trash_folder);
    }
    // after copy close the mailbox to get in unselected state
    sqimap_run_command($imap_stream,'CLOSE',false,$response,$message);
    // se ho figli rilancio me stesso sui figli
    if ($tree[$index]['doIHaveChildren']) {
        for ($j = 0;$j < count($tree[$index]['subNodes']); $j++) {
            walkTreeInPostOrderMoveMessagesUnderTrash($tree[$index]['subNodes'][$j], $imap_stream, $tree, $topFolderName);
	}
    }
}

function walkTreeInPostOrderDeleteMessagesUnderFolder($index, $imap_stream, $tree, $topFolderName) {
    global $trash_folder, $delimiter, $default_folder_prefix;
    $mbx_response = sqimap_mailbox_select($imap_stream, $tree[$index]['value']);
    $messageCount = $mbx_response['EXISTS'];
    if ($messageCount > 0) {
        sqimap_messages_flag($imap_stream, 1, '*', 'Deleted', true);
    }
    // after copy close the mailbox to get in unselected state
    sqimap_run_command($imap_stream,'CLOSE',false,$response,$message);
    // se ho figli rilancio me stesso sui figli
    if ($tree[$index]['doIHaveChildren']) {
        for ($j = 0;$j < count($tree[$index]['subNodes']); $j++) {
            walkTreeInPostOrderDeleteMessagesUnderFolder($tree[$index]['subNodes'][$j], $imap_stream, $tree, $topFolderName);
	}
    }
}

function simpleWalkTreePre($index, $tree) {
    if ($tree[$index]['doIHaveChildren']) {
        for ($j = 0; $j < count($tree[$index]['subNodes']); $j++) {
            simpleWalkTreePre($tree[$index]['subNodes'][$j], $tree);
        }
        echo codeHTML($tree[$index]['value']) . '<br>';
    } else {
        echo codeHTML($tree[$index]['value']) . '<br>';
    }
}
?>
