<?php

/********************************************************************
 *   Autori:             Marco Colombo (marco@openit.it)
 *                       Valentina Portolan (valentina@openit.it)
 *                       Giuliano Natali Diaolin (diaolin@openit.it)
 *  Copyright:           Open It S.r.l.
 *                       Viale Dante, 78
 *                       38057 Pergine Valsugana (TN) ITALY
 *                       Tel: +39 0461 534800 Fax: +39 0461 538443
 *********************************************************************/

/**
 * Message and Spam Filter Plugin
 *
 * Copyright (c) 1999-2003 The SquirrelMail Project Team
 * Licensed under the GNU GPL. For full terms see the file COPYING.
 *
 * This plugin filters your inbox into different folders based upon given
 * criteria.  It is most useful for people who are subscibed to mailing lists
 * to help organize their messages.  The argument stands that filtering is
 * not the place of the client, which is why this has been made a plugin for
 * SquirrelMail.  You may be better off using products such as Sieve or
 * Procmail to do your filtering so it happens even when SquirrelMail isn't
 * running.
 *
 * If you need help with this, or see improvements that can be made, please
 * email me directly at the address above.  I definately welcome suggestions
 * and comments.  This plugin, as is the case with all SquirrelMail plugins,
 * is not directly supported by the developers.  Please come to me off the
 * mailing list if you have trouble with it.
 *
 * Also view plugins/README.plugins for more information.
 *
 * $Id: filters.php,v 1.3 2004/03/26 08:29:32 marco Exp $
 */

if (!defined('SM_PATH')) { define('SM_PATH','../../'); }

function start_filters($ox="") {
    global $imapServerAddress, $imapPort, $imap, $default_folder_prefix, $sent_folder,
           $imap_general, $filters, $imap_stream, $imapConnection,
           $UseSeparateImapConnection;

    sqgetGlobalVar('username', $username, SQ_SESSION);
    sqgetGlobalVar('key',      $key,      SQ_COOKIE);

        // Detect if we have already connected to IMAP or not.
        // Also check if we are forced to use a separate IMAP connection
        if ((!isset($imap_stream) && !isset($imapConnection)) ||
            $UseSeparateImapConnection) {
                $stream = sqimap_login($username, $key, $imapServerAddress,
                                    $imapPort, 10);
                $previously_connected = false;
        } elseif (isset($imapConnection)) {
            $stream = $imapConnection;
            $previously_connected = true;
        } else {
            $previously_connected = true;
            $stream = $imap_stream;
        }
	if ($ox) {
    	    if (sqimap_get_num_messages($stream, $ox) > 0) {
        	// Sort into folders
        	user_filters($stream,$ox);
    	    }
	} else {
    	    if (sqimap_get_num_messages($stream, 'INBOX') > 0) {
        	// Sort into folders
        	user_filters($stream,'INBOX');
    	    }
	}

        if (!$previously_connected) {
            sqimap_logout($stream);
        }
}

function user_filters($imap_stream,$mailbox) {
    global $data_dir, $username, $sent_folder;
    $filters = load_filters();
    if (!$filters) { return; }
    $filters_sent_folder = getPref($data_dir, $username, 'filters_sent_folder');
    if ($mailbox == $sent_folder && !$filters_sent_folder) { return; }
    if ($mailbox == $sent_folder) {
	$filters_user_scan = "";
    } else {
	$filters_user_scan = getPref($data_dir, $username, 'filters_user_scan');
    }
    sqimap_mailbox_select($imap_stream, $mailbox);
    $id = array();
    // For every rule
    for ($i=0, $num = count($filters); $i < $num; $i++) {
        // If it is the "combo" rule
        if ($filters[$i]['where'] == 'To or Cc') {
            /*
            *  If it's "TO OR CC", we have to do two searches, one for TO
            *  and the other for CC.
            */
            $id = filter_search_and_delete($imap_stream, 'TO',
                  $filters[$i]['what'], $filters[$i]['folder'], $filters_user_scan, $id);
            $id = filter_search_and_delete($imap_stream, 'CC',
                  $filters[$i]['what'], $filters[$i]['folder'], $filters_user_scan, $id);
        } else {
            /*
            *  If it's a normal TO, CC, SUBJECT, or FROM, then handle it
            *  normally.
            */
            $id = filter_search_and_delete($imap_stream, $filters[$i]['where'],
                 $filters[$i]['what'], $filters[$i]['folder'], $filters_user_scan, $id);
        }
    }
    // Clean out the mailbox whether or not auto_expunge is on
    // That way it looks like it was redirected properly
    if (count($id)) {
        sqimap_mailbox_expunge($imap_stream, $mailbox);
    }
}

function filter_search_and_delete($imap, $where, $what, $where_to, $user_scan, 
                                  $del_id) {
    global $languages, $squirrelmail_language, $allow_charset_search, 
           $uid_support, $imap_server_type;
    if ($user_scan == 'new') {
        $category = 'UNSEEN';
    } else {
        $category = 'ALL';
    }

    if ($allow_charset_search &&
        isset($languages[$squirrelmail_language]['CHARSET']) &&
        $languages[$squirrelmail_language]['CHARSET']) {
        $search_str = 'SEARCH CHARSET '
                    . strtoupper($languages[$squirrelmail_language]['CHARSET'])
                    . ' ' . $category;
    } else {
        $search_str = 'SEARCH CHARSET US-ASCII ' . $category;
    }
    if ($where == 'Header') {
        $what  = explode(':', $what);
        $where = trim($where . ' ' . $what[0]);
        $what  = addslashes(trim($what[1]));
    }

    if ($imap_server_type == 'macosx') {    
	$search_str .= ' ' . $where . ' ' . $what;
    } else {
	$search_str .= ' ' . $where . ' {' . strlen($what) . "}\r\n"
                    . $what . "\r\n";
    }

    /* read data back from IMAP */
    $read = sqimap_run_command($imap, $search_str, true, $response, $message, $uid_support);

    // This may have problems with EIMS due to it being goofy

    for ($r=0, $num = count($read); $r < $num &&
                substr($read[$r], 0, 8) != '* SEARCH'; $r++) {}
    if ($response == 'OK') {
        $ids = explode(' ', $read[$r]);
        if (sqimap_mailbox_exists($imap, $where_to)) {
            for ($j=2, $num = count($ids); $j < $num; $j++) {
                $id = trim($ids[$j]);
                $del_id[] = $id;
                sqimap_messages_copy ($imap, $id, $id, $where_to);
                sqimap_messages_flag ($imap, $id, $id, 'Deleted',false);
            }
        }
    }
    return $del_id;
}

function load_filters() {
    global $data_dir, $username;

    $filters = array();
    $i=0;
    while($fltr = getPref($data_dir, $username, 'filter' . $i)) {
	    $ary = explode(',', $fltr);
    	    $filters[$i]['where'] = $ary[0];
    	    $filters[$i]['what'] = $ary[1];
    	    $filters[$i]['folder'] = $ary[2];
	    $i++;
    }
    return $filters;
}

function remove_filter ($id) {
    global $data_dir, $username;

    while ($nextFilter = getPref($data_dir, $username, 'filter' .
        ($id + 1))) {
        setPref($data_dir, $username, 'filter' . $id, $nextFilter);
        $id ++;
    }

    removePref($data_dir, $username, 'filter' . $id);
}

function filter_swap($id1, $id2) {
    global $data_dir, $username;

    $FirstFilter = getPref($data_dir, $username, 'filter' . $id1);
    $SecondFilter = getPref($data_dir, $username, 'filter' . $id2);

    if ($FirstFilter && $SecondFilter) {
        setPref($data_dir, $username, 'filter' . $id2, $FirstFilter);
        setPref($data_dir, $username, 'filter' . $id1, $SecondFilter);
    }
}

/* This update the filter rules when
   renaming or deleting folders */
function update_for_folder ($args) {
    $old_folder = $args[0];
        $new_folder = $args[2];
        $action = $args[1];
    global $plugins, $data_dir, $username;
    $filters = array();
    $filters = load_filters();
    $filter_count = count($filters);
    $p = 0;
    for ($i=0;$i<$filter_count;$i++) {
        if (!empty($filters)) {
            if ($old_folder == $filters[$i]['folder']) {
                if ($action == 'rename') {
                    $filters[$i]['folder'] = $new_folder;
                    setPref($data_dir, $username, 'filter'.$i,
                    $filters[$i]['where'].','.$filters[$i]['what'].','.$new_folder);
                }
                elseif ($action == 'delete') {
                    remove_filter($p);
                    $p = $p-1;
                }
            }
        $p++;
        }
    }
}

?>
