<?php

/********************************************************************
 *   Autori:             Marco Colombo (marco@openit.it)
 *                       Valentina Portolan (valentina@openit.it)
 *                       Giuliano Natali Diaolin (diaolin@openit.it)
 *  Copyright:           Open It S.r.l.
 *                       Viale Dante, 78
 *                       38057 Pergine Valsugana (TN) ITALY
 *                       Tel: +39 0461 534800 Fax: +39 0461 538443
 *********************************************************************/

/**
 * tree_move_mail.php
 *
 * Copyright (c) 1999-2003 The SquirrelMail Project Team
 * Licensed under the GNU GPL. For full terms see the file COPYING.
 *
 * This is the code for the left bar. The left bar shows the folders
 * available, and has cookie information.
 *
 * $Id: tree_folders.php,v 1.3 2004/06/01 08:37:55 marco Exp $
 */

/* Path for SquirrelMail required files. */
define('SM_PATH','../');

/* SquirrelMail required files. */
require_once(SM_PATH . 'include/validate.php');
require_once(SM_PATH . 'functions/imap.php');
require_once(SM_PATH . 'functions/plugin.php');
require_once(SM_PATH . 'functions/page_header.php');
require_once(SM_PATH . 'functions/html.php');

/* These constants are used for folder stuff. */
define('SM_BOX_UNCOLLAPSED', 0);
define('SM_BOX_COLLAPSED',   1);

/* --------------------- FUNCTIONS ------------------------- */

/*
 * Returns sorted mailbox lists in several different ways. 
 * See comment on sqimap_mailbox_parse() for info about the returned array.
 */
function sqimap_mailbox_list_overlook($imap_stream, $force=false) {
    global $default_folder_prefix;

    if (!sqgetGlobalVar('boxesnew',$boxesnew,SQ_SESSION) || $force) {
        global $data_dir, $username, $list_special_folders_first,
               $folder_prefix, $trash_folder, $sent_folder, $draft_folder,
               $move_to_trash, $move_to_sent, $save_as_draft,
               $delimiter, $noselect_fix_enable;
        $inbox_in_list = false;
        $inbox_subscribed = false;

        require_once(SM_PATH . 'include/load_prefs.php');

        if ($noselect_fix_enable) {
            $lsub_args = "LIST \"$folder_prefix\" \"*%\"";
        } else {
            $lsub_args = "LIST \"$folder_prefix\" \"*\"";
        }
        /* LSUB array */
        $lsub_ary = sqimap_run_command ($imap_stream, $lsub_args,
                                        true, $response, $message);

        $sorted_lsub_ary = array();
        for ($i = 0, $cnt = count($lsub_ary);$i < $cnt; $i++) {
            /*
             * Workaround for mailboxes returned as literal
             * Doesn't work if the mailbox name is multiple lines 
	     * (larger then fgets buffer)
             */
            if (isset($lsub_ary[$i + 1]) && substr($lsub_ary[$i],-3) == "}\r\n") {
		if (ereg("^(\\* [A-Z]+.*)\\{[0-9]+\\}([ \n\r\t]*)$",
                     $lsub_ary[$i], $regs)) {
            	    $i++;
            	    $lsub_ary[$i] = $regs[1] . '"' . addslashes(trim($lsub_ary[$i])) . '"' . $regs[2];
		}
            }
            $temp_mailbox_name = find_mailbox_name($lsub_ary[$i]);
            $sorted_lsub_ary[] = $temp_mailbox_name;
            if (!$inbox_subscribed && strtoupper($temp_mailbox_name) == 'INBOX') {
                $inbox_subscribed = true;
            }
        }
	/* remove duplicates */
	$sorted_lsub_ary = array_unique($sorted_lsub_ary);

	/* natural sort mailboxes */
        if (isset($sorted_lsub_ary)) {
            usort($sorted_lsub_ary, 'user_strcasecmp');
        }
	/*
	 * The LSUB response doesn't provide us information about \Noselect
	 * mail boxes. The LIST response does, that's why we need to do a LIST
	 * call to retrieve the flags for the mailbox
           * Note: according RFC2060 an imap server may provide \NoSelect flags in the LSUB response.
           * in other words, we cannot rely on it.
	 */
        $sorted_list_ary = array();
        for ($i=0; $i < count($sorted_lsub_ary); $i++) {
            if (substr($sorted_lsub_ary[$i], -1) == $delimiter) {
                $mbx = substr($sorted_lsub_ary[$i], 0, strlen($sorted_lsub_ary[$i])-1);
            }
            else {
                $mbx = $sorted_lsub_ary[$i];
            }

            $read = sqimap_run_command ($imap_stream, "LIST \"\" \"$mbx\"",
                                        true, $response, $message);

            /* Another workaround for literals */

            if (isset($read[1]) && substr($read[1],-3) == "}\r\n") {
		if (ereg("^(\\* [A-Z]+.*)\\{[0-9]+\\}([ \n\r\t]*)$",
                     $read[0], $regs)) {
            	    $read[0] = $regs[1] . '"' . addslashes(trim($read[1])) . '"' . $regs[2];
		}
            }

            if (isset($read[0])) {
                $sorted_list_ary[$i] = $read[0];
            } else {
                $sorted_list_ary[$i] = '';
            }
        }

        /*
         * Just in case they're not subscribed to their inbox,
         * we'll get it for them anyway
         */
        if (!$inbox_subscribed) {
            $inbox_ary = sqimap_run_command ($imap_stream, "LIST \"\" \"INBOX\"",
                                             true, $response, $message);
            /* Another workaround for literals */
            if (isset($inbox_ary[1]) && substr($inbox_ary[0],-3) == "}\r\n") {
		if (ereg("^(\\* [A-Z]+.*)\\{[0-9]+\\}([ \n\r\t]*)$",
                     $inbox_ary[0], $regs)) {
            	    $inbox_ary[0] = $regs[1] . '"' . addslashes(trim($inbox_ary[1])) .
                                '"' . $regs[2];
		}
            }
            $sorted_list_ary[] = $inbox_ary[0];
            $sorted_lsub_ary[] = find_mailbox_name($inbox_ary[0]);
        }

        $boxesall = sqimap_mailbox_parse ($sorted_list_ary, $sorted_lsub_ary);
	
        /* Now, lets sort for special folders */
        $boxesnew = $used = array();

        /* Find INBOX */
        $cnt = count($boxesall);
	$used = array_pad($used,$cnt,false);
        for($k = 0; $k < $cnt; ++$k) {
            if (strtolower($boxesall[$k]['unformatted']) == 'inbox') {
                $boxesnew[] = $boxesall[$k];
                $used[$k] = true;
		break;
            }
        }
        /* List special folders and their subfolders, if requested. */
        if ($list_special_folders_first) {
            for($k = 0; $k < $cnt; ++$k) {
                if (!$used[$k] && isSpecialMailbox($boxesall[$k]['unformatted'])) {
                    $boxesnew[] = $boxesall[$k];
                    $used[$k]   = true;
                }
	    }
	}

        /* Rest of the folders */
        for($k = 0; $k < $cnt; $k++) {
            if (!$used[$k]) {
                $boxesnew[] = $boxesall[$k];
            }
        }
    }
    return $boxesnew;
}

function formatMailboxName($imapConnection, $box_array, $id, $count) {

    global $folder_prefix, $trash_folder, $sent_folder,
           $color, $move_to_sent, $move_to_trash,
           $unseen_notify, $unseen_type, $collapse_folders,
           $draft_folder, $save_as_draft,
           $use_special_folder_color;
    $real_box = $box_array['unformatted'];
    $mailbox = str_replace('&nbsp;','',$box_array['formatted']);
    $mailboxURL = urlencode($real_box);

    /* Strip down the mailbox name. */
    if (ereg("^( *)([^ ]*)$", $mailbox, $regs)) {
        $mailbox = $regs[2];
    }

    /* Start off with a blank line. */
    $line = '';

    /* Create the link for this folder. */
    if (!isset($status) || (isset($status) && $status != false)) {
	$line = '<a name="folder'.$id.'"  id="folder'.$id.'" onMouseOut="this.style.cursor=\'normal\'" onMouseOver="this.style.cursor=\'pointer\'"
			onClick="parent.document.cf.subfolder.value=\''.$real_box.'\'; SelectedFolder('.$id.')">';
    } 
    if ( $mailbox == 'INBOX' ) {
        $line .= _("INBOX");
    } else {
        $line .= str_replace(' ','&nbsp;',$mailbox);
    }
    if (!isset($status) || (isset($status) && $status != false)) {
	$line .= '</a>';
    }

    /* Return the final product. */
    return ($line);
}

/**
 * Recursive function that computes the collapsed status and parent
 * (or not parent) status of this box, and the visiblity and collapsed
 * status and parent (or not parent) status for all children boxes.
 */
function compute_folder_children(&$parbox, $boxcount) {
    global $boxes, $data_dir, $username, $collapse_folders;
    $nextbox = $parbox + 1;

    /* Retreive the name for the parent box. */
    $parbox_name = $boxes[$parbox]['unformatted'];

    /* 'Initialize' this parent box to childless. */
    $boxes[$parbox]['parent'] = FALSE;

    $collapse = SM_BOX_UNCOLLAPSED;
    $boxes[$parbox]['collapse'] = $collapse;

    /* Otherwise, get the name of the next box. */
    if (isset($boxes[$nextbox]['unformatted'])) {
        $nextbox_name = $boxes[$nextbox]['unformatted'];
    } else {
        $nextbox_name = '';
    }

    /* Compute any children boxes for this box. */
    while (($nextbox < $boxcount) &&
           (is_parent_box($boxes[$nextbox]['unformatted'], $parbox_name))) {

        /* Note that this 'parent' box has at least one child. */
        $boxes[$parbox]['parent'] = TRUE;

        /* Compute the visiblity of this box. */
        $boxes[$nextbox]['visible'] = ($boxes[$parbox]['visible'] &&
                                       ($boxes[$parbox]['collapse'] != SM_BOX_COLLAPSED));

        /* Compute the visibility of any child boxes. */
        compute_folder_children($nextbox, $boxcount);
    }

    /* Set the parent box to the current next box. */
    $parbox = $nextbox;
}

/**
 * Create the link for a parent folder that will allow that
 * parent folder to either be collapsed or expaned, as is
 * currently appropriate.
 */
function create_collapse_link($boxnum) {
    global $boxes, $base_uri, $imapConnection, $unseen_notify, $color, $squirrelmail_plugin_hooks;
    $mailbox = urlencode($boxes[$boxnum]['unformatted']);
        
    $link = '<a href="'.$base_uri.'src/tree_folders.php?force_refresh=TRUE&';
    if ($boxes[$boxnum]['collapse'] == SM_BOX_COLLAPSED) {
        $link .= "unfold=$mailbox\">";
        $link = do_hook_function('icon_plus_minus',array($link,1));
    } else {
        $link .= "fold=$mailbox\">";
        $link = do_hook_function('icon_plus_minus',array($link,0));
    }
    $link .= '</a>';
    /* Return the finished product. */
    return ($link);
}

/**
 * This simple function checks if a box is another box's parent.
 */
function is_parent_box($curbox_name, $parbox_name) {
    global $delimiter;

    /* Extract the name of the parent of the current box. */
    $curparts = explode($delimiter, $curbox_name);
    $curname = array_pop($curparts);
    $actual_parname = implode($delimiter, $curparts);
    $actual_parname = substr($actual_parname,0,strlen($parbox_name));

    /* Compare the actual with the given parent name. */
    return ($parbox_name == $actual_parname);
}

/* -------------------- MAIN ------------------------ */

/* get globals */
sqgetGlobalVar('username', $username, SQ_SESSION);
sqgetGlobalVar('key', $key, SQ_COOKIE);
sqgetGlobalVar('delimiter', $delimiter, SQ_SESSION);
sqgetGlobalVar('onetimepad', $onetimepad, SQ_SESSION);

sqgetGlobalVar('fold', $fold, SQ_GET);
sqgetGlobalVar('unfold', $unfold, SQ_GET);

/* end globals */

// open a connection on the imap port (143)
//$imapConnection = sqimap_login($username, $key, $imapServerAddress, $imapPort, 10); // the 10 is to hide the output
$imapConnection = sqimap_login($username, $key, $imapServerAddress, $imapPort,0);

/**
 * Using stristr since older preferences may contain "None" and "none".
 */

$xtra = '';
$xtra .= '<SCRIPT language="javascript">
	var msgsubscribe = "'.jsspecialchars(_("Subscribe")).'";
	var msgunsubscribe = "'.jsspecialchars(_("Unsubscribe")).'";
	var msgsetspecial = "'.jsspecialchars(_("Set as special")).'";
	var msgunsetspecial = "'.jsspecialchars(_("Unset as special")).'";
    </SCRIPT>';

displayHtmlHeader( '', $xtra );

set_colors_js_IEbug("A:link","A.selectedfolder");

echo '<script language="javascript" type="text/javascript" src="'.$base_uri.'js/tree_folders.js"></script>';

/* If requested and not yet complete, attempt to autocreate folders. */
if ($auto_create_special && !isset($auto_create_done)) {
    $autocreate = array($sent_folder, $trash_folder, $draft_folder);
    foreach( $autocreate as $folder ) {
        if (($folder != '') && ($folder != 'none')) {
            if ( !sqimap_mailbox_exists($imapConnection, $folder)) {
                sqimap_mailbox_create($imapConnection, $folder, '');
            } else if (!sqimap_mailbox_is_subscribed($imapConnection, $folder)) {
                sqimap_subscribe($imapConnection, $folder);
            }
        }
    }

    /* Let the world know that autocreation is complete! Hurrah! */
    $auto_create_done = TRUE;
    sqsession_register($auto_create_done, 'auto_create_done');
}

sqgetGlobalVar('force_refresh',$force_refresh,SQ_GET);
// lista completa
$boxes = sqimap_mailbox_list_overlook($imapConnection,$force_refresh);
// lista sottoscritte
$boxes_normal = sqimap_mailbox_list($imapConnection,$force_refresh);
 
/* Prepare do do out collapsedness and visibility computation. */
$curbox = 0;
$boxcount = count($boxes);

$box_array=$boxes[0];
$real_box = $box_array['unformatted'];
$mailbox = str_replace('&nbsp;','',$box_array['formatted']);
$mailboxURL = urlencode($real_box);

echo "\n<body onload='parent.document.cf.subfolder.value=\"".$real_box."\"; SelectedFolderDefault(); self.focus();'>\n";

echo html_tag( 'table', '', 'left', '', 'width="100%" height="100%" valign="top" border="0" cellspacing="0" cellpadding="0"' ); 
echo html_tag( 'tr' ) . 
    html_tag( 'td', '', 'left', '', 'valign="top" width="100%" height="100%"' );

/* Compute the collapsedness and visibility of each box. */
while ($curbox < $boxcount) {
    $boxes[$curbox]['visible'] = TRUE;
    compute_folder_children($curbox, $boxcount);
}

$special_mailboxs = array();

$boxnum=0;
while ($m = getPref($data_dir, $username, 'user_special_mailboxes' . $boxnum)) {
    $special_mailboxs[$boxnum] = $m;
    $boxnum++;
}

for ($i = 0; $i < count($boxes); $i++) {
    if ( $boxes[$i]['visible'] ) {
	$subscribed_folder=0;
	for ($p = 0; $p < count($boxes_normal); $p++) {  
	    if ($boxes[$i]['unformatted'] == $boxes_normal[$p]['unformatted']) {
		$subscribed_folder=1;
		break;
	    }
	}
        $mailbox = $boxes[$i]['formatted'];
        $mblevel = substr_count($boxes[$i]['unformatted'], $delimiter) + 1;

        /* Create the prefix for the folder name and link. */
        $prefix = str_repeat('  ',$mblevel);
        $prefix = str_replace(' ','&nbsp;',substr($prefix,0,strlen($prefix)-2)).
                       do_hook_function('icon_plus_minus',array($link,0))."&nbsp;";
	$prefix = do_hook_function('icon_folder',array($prefix,$mailbox)); 
        $line = "<nobr><tt>$prefix</tt>";
	if (!$subscribed_folder) { $line .= '<span id="unsub'.$i.'" class="unsubscribe" >'; }
	else if (in_array($boxes[$i]['unformatted'],$special_mailboxs)) { $line .= '<span id="special'.$i.'" class="specialfolder" >'; }
        if (in_array('noselect', $boxes[$i]['flags'])) {
            if (ereg("^( *)([^ ]*)", $mailbox, $regs)) {
                $mailbox = str_replace('&nbsp;','',$mailbox);
                $line .= str_replace(' ', '&nbsp;', $mailbox);
            }
        } else {
            $line .= formatMailboxName($imapConnection, $boxes[$i], $i, count($boxes));
        }

	if (!$subscribed_folder || in_array($boxes[$i]['unformatted'],$special_mailboxs)) { $line .= '</span>'; }

        /* Put the final touches on our folder line. */
        $line .= "</nobr><br>\n";

        /* Output the line for this folder. */
        echo $line;
    }
}

sqimap_logout($imapConnection);

echo '</td></tr></table>';
echo "</body></html>\n";

?>
